/* Translation table for debian sections
 * (c) 2004 Michael Vogt
 */

#ifndef _HAVE_SECTIONS_TRANS_H
#define _HAVE_SECTIONS_TRANS_H

#include <string>
#include <sstream>
#include <iostream>
#include <apt-pkg/strutl.h>
#include "i18n.h"

using namespace std;

string trans_section(string sec);

#endif
